package app.service;

import java.util.Collection;
import java.util.Collections;
import java.util.regex.Matcher;

import org.fxmisc.richtext.model.StyleSpans;
import org.fxmisc.richtext.model.StyleSpansBuilder;

import app.model.ProgrammingLanguage;

public class LanguageOperations {
  
  private static String getMatch(Matcher m, Collection<String> kws) {
    return kws
      .stream()
      .filter(keyword -> m.group(keyword) != null)
      .findFirst()
      .orElse(""); // This is not possible, but is needed to convert Optional<String> to String
  }

  public static StyleSpans<Collection<String>> 
    syntaxHighlight(String text, ProgrammingLanguage language) {
    Matcher matcher = language.getPattern().matcher(text);
    StyleSpansBuilder<Collection<String>> spansBuilder = new StyleSpansBuilder<>();

    int lastKwEnd = 0;
    while(matcher.find()) {
      String styleClass = getMatch(matcher, language.getPatternMap().values());

      spansBuilder.add(
        Collections.emptyList(),
        matcher.start() - lastKwEnd
      );
      spansBuilder.add(
        Collections.singleton(styleClass),
        matcher.end() - matcher.start()
      );

      lastKwEnd = matcher.end();
    }

    spansBuilder.add(
      Collections.emptyList(),
      text.length() - lastKwEnd
    );

    return spansBuilder.create();
  }

}
